@@ -94,15 +94,20 @@ class DiffConvert:
     and the update() method would return the commands required to change
     it to the new value.
 
-    The methods called to do the conversions are:
+    The methods called to do the conversions (documented in the methods
+    themselves) are:
 
     * add(new, *args) -- appears in the new configuration but was not
       present in the old configuration; calls update(None, new, new,
       *args) by default.
 
+    * delete(old, rem, new, *args) -- appears in the old configuration
+      but is not present in the new configuration; calls remove(old,
+      *args) by default)
+
     * remove(old, *args) -- appears in the old configuration but is not
       present in the new configuration; calls truncate(old, old, None,
-      *args) by default).
+      *args) by default.
 
     * truncate(old, rem, new, *args) -- appears in the remove
       differences but still exists in the new configuration - i.e. is
@@ -126,11 +131,15 @@ class DiffConvert:
     The 'context', 'cmd' and 'ext' parameters, below, are all tuples
     giving parts of the path through the keys of the configuration
     dictionary to match for the specific converter.  These can be
-    literal keys, or the value None, if that level of the path is a
-    wildcard match for all keys - for example, '("interface", None)' to
-    match all interfaces.  The part of the configuration dictionary
-    that matches the 'context + cmd' parts of the path will be passed
-    as the 'old/new' and 'rem/upd' parameters to the action methods.
+    literal keys, the value None, or a set.  The value None or a set
+    specify a wildcard match to any value or limited to the specified
+    set of values.  For example, '("interface", None)' to match all
+    interfaces; ("redistribute", { "connected", "static" }) will match
+    only connected or static redistributions.
+
+    The part of the configuration dictionary that matches the 'context +
+    cmd' parts of the path will be passed as the 'old/new' and 'rem/upd'
+    parameters to the action methods.
 
     The child classes can override the following values:
 
@@ -161,8 +170,9 @@ class DiffConvert:
     empty_trigger -- if this is set to True, the converter will always
     fire the named triggers, even if the converter generated no output
 
-    In addition the remove() and update() methods will likely need to be
-    implemented.
+    split_args_offset -- this is used by the split_args() method: see
+    that for more information; it defaults to 0 (causing split_args() to
+    use the natural boundary between the context and cmd/ext)
     """
 
 
@@ -172,15 +182,7 @@ class DiffConvert:
     block = None
     trigger_blocks = set()
     empty_trigger = False
-
-    # _context_args_count is used by _context_args to determine the
-    # number of arguments which should be included in the context (and
-    # the rest in the 'remaining' arguments)
-    #
-    # see the __init__() constructor for more information on how they're
-    # used
-    _context_args_count = None
-    _context_args_offset = 0
+    split_args_offset = 0
 
 
     def __init__(self):
@@ -210,42 +212,44 @@ class DiffConvert:
         # need these to get the argument list to pass to the converter
         # action methods)
         self.wildcard_indices = [
-            i for i, v in enumerate(self._path_full) if v is None ]
+            i for i, v in enumerate(self._path_full)
+                if (v is None) or isinstance(v, set) ]
 
-        # initialise _context_args_count if it isn't explicitly defined
-        # (the default is None, which means 'undefined')
+        # set _context_args value for this converter, which is used by
+        # the split_args() method to vary the split between the
+        # arguments used in the context vs those in the cmd and ext
+        # elements returned by that function
         #
-        # the calculated default is usually fine but can be overridden -
-        # this is sometimes useful when writing classes covering the
-        # same commands, shared across different contexts and the
-        # context must be dynamically processed
+        # this is useful when writing converters which can work in
+        # several different contexts (such as those for commands in BGP
+        # configuration)
         #
-        # an example would be an enter() method which takes the wildcard
-        # arguments from the context and is overridden in a child class
-        if self._context_args_count is None:
-            # calculate the number of wildcard arguments in the context
-            # (elements which are None)
-            context_wildcard_count = len(
-               [ i for i, v in enumerate(self.context) if v is None ])
-
-            # the count is the number of wildcard arguments minus the
-            # 'offset' - the offset defaults to 0 so is the same but
-            # can be overridden to shift it
-            #
-            # the offset is useful when parameters need to be added or
-            # removed from this list and subclasses are generated where
-            # the context is changed by the offset remains consistent
-            self._context_args_count = (
-                context_wildcard_count + self._context_args_offset)
+        # if set to negative values, additional wildcard arguments from
+        # the context will be included in the cmd/ext side; positive
+        # values will shift the other way
+        self._context_args = (
+            len([ i for i, v in enumerate(self.context) if v is None ])
+            + self.split_args_offset)
 
 
-    def _context_args(self, args):
+    def split_args(self, args):
         """Split a list of wildcard arguments into a tuple - those used
-        in the context vs those used afterwards.
+        in the context vs those used afterwards (by the cmd and ext
+        parts).
+
+        This is useful when writing converters which can work in several
+        different contexts (such as those for commands in BGP
+        configuration).
+
+        If set to negative values, additional wildcard arguments from
+        the context will be included in the cmd/ext side; positive
+        values will shift the other way.
+
+        The boundary between these can be adjusted from what would
+        automatically be inferred by setting split_args_offset.
         """
 
-        return (args[:self._context_args_count],
-                args[self._context_args_count:])
+        return args[:self._context_args], args[self._context_args:]
 
 
     def _path_matches(self, d, path):
@@ -274,27 +278,28 @@ class DiffConvert:
             return []
 
         # if the path element is None, we're doing a wildcard match
-        if path_head is None:
+        if (path_head is None) or (isinstance(path_head, set)):
             # initialise an empty list for all returned matching
             # results
             results = []
 
             # go through all the keys at this level in the dictonary
             for d_key in d:
-                # are there levels below this one?
-                if path_tail:
-                    # yes - recursively get the matches from under
-                    # this key
-                    for matches in self._path_matches(d[d_key], path_tail):
-                        # add this match to the list of matches,
-                        # prepending this key onto the start of the
-                        # path
-                        results.append([d_key] + matches)
-                else:
-                    # no - just add this result (this is a minor
-                    # optimisation, as well as avoiding an error by
-                    # trying to index into a non-dict type), above
-                    results.append([d_key])
+                if (path_head is None) or (d_key in path_head):
+                    # are there levels below this one?
+                    if path_tail:
+                        # yes - recursively get the matches from under
+                        # this key
+                        for matches in self._path_matches(d[d_key], path_tail):
+                            # add this match to the list of matches,
+                            # prepending this key onto the start of the
+                            # path
+                            results.append([d_key] + matches)
+                    else:
+                        # no - just add this result (this is a minor
+                        # optimisation, as well as avoiding an error by
+                        # trying to index into a non-dict type), above
+                        results.append([d_key])
 
             return results
 
@@ -326,12 +331,13 @@ class DiffConvert:
 
         return self._path_matches(d, self._path_full)
 
+
     def explicit_context_matches(self, d, context):
         """This method is used when checking for trigger matches.
 
         It is similar to full_matches() except the explicit context
-        (without wildcards) is supplied and the command path appended;
-        the matches are returned as a list.
+        (without wildcards) is supplied and the command and extended
+        paths appended; the matches are returned as a list.
 
         The context will have been taken from the converter which set up
         the trigger (with the wildcarded fields completed) so we only
@@ -339,7 +345,7 @@ class DiffConvert:
         explicit context.
         """
 
-        return self._path_matches(d, tuple(context) + self.cmd)
+        return self._path_matches(d, tuple(context) + self.cmd + self.ext)
 
 
     def context_removed(self, d, match):
@@ -484,6 +490,34 @@ class DiffConvert:
         return [ match[i] for i in self.wildcard_indices ]
 
 
+    def delete(self, old, rem, new, *args):
+        """The delete() method is called when the specified path is in
+        the remove differences (i.e. it's in the old configuration but
+        but not in the new configuration), unless the containing context
+        is being removed in its entirety.
+
+        The default behaviour of this method is to call remove() with
+        the old value for 'old'.
+
+        This method may be required in odd situations where there is a
+        full removal of a configuration element matched by the 'cmd' and
+        'ext' attributes but there still something in the context (at a
+        higher level) and this is required to do the removal.  Normally
+        the remove() method should be sufficient, however.
+
+        The difference between delete() and truncate() is that the
+        latter is called when something remains in the configuration
+        value identified by the complete match (i.e. context + cmd +
+        ext), so only some items are being removed.  delete(), on the
+        other hand, is called when the configuration item identified by
+        the match is empty; 'rem' and 'new' are identify the
+        configuration identified by just the context and not the
+        complete match and may be required when deleting items.
+        """
+
+        return self.remove(old, *args)
+
+
     def remove(self, old, *args):
         """The remove() method is called when the specified path is in
         the remove differences (i.e. it's in the old configuration but
@@ -727,8 +761,8 @@ class DiffConfig:
         # name, with each value a list of converters in that block, to
         # be applied in the specified order
         self._cvts = {}
-        for b in self._blocks:
-            self._cvts[b] = []
+        for block in self._blocks:
+            self._cvts[block] = []
 
         # add and sort the converters
         self._add_converters()
@@ -1006,9 +1040,16 @@ class DiffConfig:
         """Print the supplied debugging message followed by a blank
         line, if it's not empty (in which case, do nothing), unless
         blank_if_single is True and the message is only one line long.
+
+        Prior to printing the message, _print_debug_block() and
+        _print_debug_converter() will be called, to print those
+        messages, if they haven't already been printed.
         """
 
         if msg:
+            self._print_debug_block()
+            self._print_debug_converter()
+
             for line in msg:
                 print(line)
 
@@ -1016,6 +1057,48 @@ class DiffConfig:
                 print()
 
 
+    def _print_debug_block(self):
+            """Print the block debug message explicitly, if the debug
+            level is above DEBUG_CONVERT_MATCH (the bottom level).
+
+            Normally this is called by _print_debug() when the first
+            debug message is displayed but, if more detailed debugging
+            is requireed, it can be called explicitly.
+
+            After printing, the message is cleared to not be printed
+            again.
+            """
+
+            if self._debug_block_msg:
+                for line in self._debug_block_msg:
+                    print(line)
+
+                print()
+
+                self._debug_block_msg = []
+
+
+    def _print_debug_converter(self):
+            """Print the converter debug message explicitly, if the debug
+            level is above DEBUG_CONVERT_MATCH (the bottom level).
+
+            Normally this is called by _print_debug() when the first
+            debug message is displayed but, if more detailed debugging
+            is requireed, it can be called explicitly.
+
+            After printing, the message is cleared to not be printed
+            again.
+            """
+
+            if self._debug_converter_msg:
+                for line in self._debug_converter_msg:
+                    print(line)
+
+                print()
+
+                self._debug_converter_msg = []
+
+
     def convert(self, old_cfg, new_cfg):
         """This method processes the conversion from the old
         configuration to the new configuration, removing excluded parts
@@ -1123,12 +1206,28 @@ class DiffConfig:
 
         current_block = ""
 
+        self._debug_block_msg = []
+
 
         # go through the list of blocks and then the converter objects
         # within them, in order
 
         for cvt in chain.from_iterable(
-            [ self._cvts[s] for s in self._blocks ]):
+            [ self._cvts[block] for block in self._blocks ]):
+
+
+            # if we're entering a new block, set a 'new block' debug
+            # message and print it immediately, if we are debugging to
+            # that level
+
+            if cvt.block != current_block:
+                current_block = cvt.block
+
+                if self._debug_convert > DEBUG_CONVERT_MATCH:
+                    self._debug_block_msg = [">> [block: %s]" % current_block]
+
+                    if self._debug_convert >= DEBUG_CONVERT_NOMATCH:
+                        self._print_debug_block()
 
 
             # get all the remove and update matches for this converter
@@ -1176,25 +1275,16 @@ class DiffConfig:
             # store this message to be printed later, if so - it doesn't
             # currently seem worth doing that for debugging messages
 
-            if (self._debug_convert >= DEBUG_CONVERT_NOMATCH
-                or (all_matches and
-                    self._debug_convert >= DEBUG_CONVERT_STEPS)):
+            self._debug_converter_msg = []
 
+            if self._debug_convert > DEBUG_CONVERT_MATCH:
+                self._debug_converter_msg = [
+                    ">> " + pathstr([ "*" if i is None else i
+                                        for i in cvt.get_path() ]),
+                    "-> " + type(cvt).__name__]
 
-                # if we're entering a new block, print a message
-
-                if all_matches and (cvt.block != current_block):
-                    current_block = cvt.block
-                    print(">> [block: %s]" % current_block)
-                    print()
-
-
-                print(">>", pathstr([ "*" if i is None else i
-                                    for i in cvt.get_path() ]))
-
-                print("->", type(cvt).__name__)
-
-                print()
+                if self._debug_convert >= DEBUG_CONVERT_NOMATCH:
+                    self._print_debug_converter()
 
 
             for match in all_matches:
@@ -1260,14 +1350,18 @@ class DiffConfig:
                     cvt_new = cvt.get_cfg(new_cfg, match)
 
                     if self._debug_convert >= DEBUG_CONVERT_PARAMS:
-                        debug_msg.append(
+                        debug_msg.extend([
                             "-> old configuration:",
-                            yaml.dump(cvt_old, default_flow_style=False))
+                            yaml.dump(cvt_old, default_flow_style=False)])
 
                         if remove_is_truncate:
-                            debug_msg.append(
+                            debug_msg.extend([
                                 "-> remove configuration:",
-                                yaml.dump(cvt_rem, default_flow_style=False))
+                                yaml.dump(cvt_rem, default_flow_style=False)])
+
+                        debug_msg.extend([
+                            "-> new configuration:",
+                            yaml.dump(cvt_new, default_flow_style=False)])
 
 
                     # get elements in the path matching wildcards
@@ -1275,7 +1369,7 @@ class DiffConfig:
                     args = cvt.get_wildcard_args(match)
 
 
-                    # call the remove converter action method using the
+                    # call the delete converter action method using the
                     # corresponding entry from the old configuration
 
                     try:
@@ -1283,23 +1377,23 @@ class DiffConfig:
                             diff = cvt.truncate(
                                        cvt_old, cvt_rem, cvt_new, *args)
                         else:
-                            diff = cvt.remove(cvt_old, *args)
+                            diff = cvt.delete(
+                                       cvt_old, cvt_rem, cvt_new, *args)
 
                     except:
                         # builld a list of arguments for passing to the
                         # converter action method for the exception
                         # message
-                        action_args = ["  old=" + repr(cvt_old)]
+                        action_args_str = ["  old=" + repr(cvt_old)]
                         if remove_is_truncate:
-                            action_args.extend([
-                                "  rem=" + repr(cvt_rem),
-                                "  new=" + repr(cvt_new)])
+                            action_args_str.append("  rem=" + repr(cvt_rem))
+                        action_args_str.append("  new=" + repr(cvt_new))
 
                         print("Exception in %s.%s() with:"
                                   % (type(cvt).__name__,
                                      "truncate" if remove_is_truncate
-                                         else "remove"),
-                              *action_args,
+                                         else "delete"),
+                              *action_args_str,
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1428,15 +1522,15 @@ class DiffConfig:
 
                     if self._debug_convert >= DEBUG_CONVERT_PARAMS:
                         if not update_is_add:
-                            debug_msg.append(
+                            debug_msg.extend([
                                 "-> old configuration:",
                                 yaml.dump(cvt_old, default_flow_style=False),
                                 "-> update configuration:",
-                                yaml.dump(cvt_upd, default_flow_style=False))
+                                yaml.dump(cvt_upd, default_flow_style=False)])
 
-                        debug_msg.append(
+                        debug_msg.extend([
                             "-> new configuration:",
-                            yaml.dump(cvt_new, default_flow_style=False))
+                            yaml.dump(cvt_new, default_flow_style=False)])
 
 
                     # call the update converter action method
@@ -1453,17 +1547,17 @@ class DiffConfig:
                         # builld a list of arguments for passing to the
                         # converter action method for the exception
                         # message
-                        action_args = []
+                        action_args_str = []
                         if not update_is_add:
-                            action_args.extend([
+                            action_args_str.extend([
                                 "  old=" + repr(cvt_old),
                                 "  upd=" + repr(cvt_upd)])
-                        action_args.append("  new=" + repr(cvt_new))
+                        action_args_str.append("  new=" + repr(cvt_new))
 
                         print("Exception in %s.%s() with:"
                                   % (type(cvt).__name__,
                                      "add" if update_is_add else "update"),
-                              *action_args,
+                              *action_args_str,
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1551,9 +1645,9 @@ class DiffConfig:
                     cvt_new = cvt.get_cfg(new_cfg, match)
 
                     if self._debug_convert >= DEBUG_CONVERT_PARAMS:
-                        debug_msg.append(
+                        debug_msg.extend([
                             "-> new configuration:",
-                            yaml.dump(cvt_new, default_flow_style=False))
+                            yaml.dump(cvt_new, default_flow_style=False)])
 
 
                     # call the trigger converter action method
