@@ -39,7 +39,7 @@ def neighbor_canonicalize(nbr):
 
 
 
-# IP[V6] ROUTE ...
+# ip[v6] route ...
 
 
 
@@ -67,7 +67,7 @@ class Cmd_IPRoute(IndentedContextualCommand):
         if metric1 or metric2:
             r["metric"] = int(metric1 or metric2)
         if tag:
-            r["tag"] = tag
+            r["tag"] = int(tag)
 
         deepsetdefault(cfg, "ip-route", vrf, net)[id] = r
 
@@ -95,7 +95,7 @@ class Cmd_IPv6Route(IndentedContextualCommand):
 
 
 
-# ROUTE-MAP ...
+# route-map ...
 
 
 
@@ -135,7 +135,7 @@ class Cmd_RtMap_MatchCmty(CmdContext_RtMap):
         m = deepsetdefault(cfg, "match", "community")
         m.setdefault("communities", set()).update(cmtys.split(" "))
         if exact:
-            m.setdefault("options", set()).add("exact-match")
+            m["exact-match"] = True
 
 
 class Cmd_RtMap_MatchIPAddr(CmdContext_RtMap):
@@ -175,10 +175,10 @@ class Cmd_RtMap_SetCmty(CmdContext_RtMap):
     match = r"set community (?P<cmtys>.+?)(?P<add> additive)?"
 
     def parse(self, cfg, cmtys, add):
-        c = deepsetdefault(cfg, "set", "community", last=set())
-        if not add:
-            c.clear()
-        c.update(cmtys.split(" "))
+        s = deepsetdefault(cfg, "set", "community")
+        s.setdefault("communities", set()).update(cmtys.split(" "))
+        if add:
+            s["additive"] = True
 
 
 class Cmd_RtMap_SetIPNxtHop(CmdContext_RtMap):
@@ -195,19 +195,28 @@ class Cmd_RtMap_SetIPNxtHop(CmdContext_RtMap):
                 # the 'vrf' key is set if 'global' or a VRF is
                 # specified for the next hop - if to global, the empty
                 # string is used (we could use None but we're being
-                # consistent with 'set global'
+                # consistent with 'set global')
                 nexthop["vrf"] = vrf or ""
             l.append(nexthop)
 
 
 class Cmd_RtMap_SetIPNxtHopVrfy(CmdContext_RtMap):
+    match = r"set ip next-hop verify-availability"
+
+    def parse(self, cfg):
+        # the next-hop addresses are a list, built up in the order
+        # they're specified, optionally with a named or global VRF
+        deepsetdefault(cfg, "set")["ip-next-hop-verify-availability"] = True
+
+
+class Cmd_RtMap_SetIPNxtHopVrfyTrk(CmdContext_RtMap):
     match = (r"set ip next-hop verify-availability (?P<addr>[0-9.]+) "
              r"(?P<seq>\d+) track (?P<obj>\d+)")
 
     def parse(self, cfg, addr, seq, obj):
         # the next-hop addresses are a list, built up in the order
         # they're specified, optionally with a named or global VRF
-        v = deepsetdefault(cfg, "set", "ip-next-hop-verify-availability")
+        v = deepsetdefault(cfg, "set", "ip-next-hop-verify-availability-track")
         v[int(seq)] = {
             "addr": addr,
             "track-obj": int(obj)
@@ -215,7 +224,7 @@ class Cmd_RtMap_SetIPNxtHopVrfy(CmdContext_RtMap):
 
 
 class Cmd_RtMap_SetIPv6NxtHop(CmdContext_RtMap):
-    match = (r"set ipv6 next-hop (?P<addrs>[0-9a-f:]+)")
+    match = r"set ipv6 next-hop (?P<addrs>[0-9a-f: ]+)"
 
     def parse(self, cfg, addrs):
         # the next-hop addresses are a list, built up in the order
@@ -228,6 +237,22 @@ class Cmd_RtMap_SetIPv6NxtHop(CmdContext_RtMap):
             l.append({ "addr": addr })
 
 
+class Cmd_RtMap_SetIPv6NxtHopVrfyTrk(CmdContext_RtMap):
+    match = (r"set ipv6 next-hop verify-availability (?P<addr>[0-9a-f: ]+) "
+             r"(?P<seq>\d+) track (?P<obj>\d+)")
+
+    def parse(self, cfg, addr, seq, obj):
+        # the next-hop addresses are a list, built up in the order
+        # they're specified, optionally with a named or global VRF
+        # TODO: canonicalise address
+        v = deepsetdefault(
+                cfg, "set", "ipv6-next-hop-verify-availability-track")
+        v[int(seq)] = {
+            "addr": addr,
+            "track-obj": int(obj)
+        }
+
+
 class Cmd_RtMap_SetLocalPref(CmdContext_RtMap):
     match = r"set local-preference (?P<pref>\d+)"
 
@@ -246,7 +271,7 @@ class Cmd_RtMap_SetVRF(CmdContext_RtMap):
 
 
 
-# ROUTER BGP ...
+# router bgp ...
 
 
 
@@ -264,13 +289,6 @@ class CmdContext_RtrBGP(IndentedContextualCommand):
     context = "router-bgp"
 
 
-class Cmd_RtrBGP_BGPRtrID(CmdContext_RtrBGP):
-    match = r"bgp router-id (?P<id>\S+)"
-
-    def parse(self, cfg, id):
-        cfg["router-id"] = id
-
-
 class CmdContext_RtrBGP_NbrFallOver(CmdContext_RtrBGP):
     match = (r"neighbor (?P<nbr>\S+) fall-over"
              r" (bfd (?P<bfd>single-hop|multi-hop)|route-map (?P<rtmap>\S+))")
@@ -292,7 +310,6 @@ class CmdContext_RtrBGP_NbrPwd(CmdContext_RtrBGP):
 
 
 class CmdContext_RtrBGP_NbrPrGrp(CmdContext_RtrBGP):
-    # this class matches the creation of a peer-group
     match = r"neighbor (?P<nbr>\S+) peer-group"
 
     def parse(self, cfg, nbr):
@@ -301,7 +318,6 @@ class CmdContext_RtrBGP_NbrPrGrp(CmdContext_RtrBGP):
 
 
 class CmdContext_RtrBGP_NbrPrGrpMbr(CmdContext_RtrBGP):
-    # this class matches the addition of a member to a peer-group
     match = r"neighbor (?P<nbr>\S+) peer-group (?P<grp>\S+)"
 
     def parse(self, cfg, nbr, grp):
@@ -330,6 +346,8 @@ class CmdContext_RtrBGP_NbrUpdSrc(CmdContext_RtrBGP):
 
 
 class Cmd_RtrBGP_AF(CmdContext_RtrBGP):
+    # this regexp will match 'vpnv[46] vrf ...' which is illegal, but we're
+    # not trying to validate commands
     match = (r"address-family (?P<af>ipv[46]( (?P<cast>unicast|multicast))?|"
              r"vpnv4|vpnv6)( vrf (?P<vrf>\S+))?")
 
@@ -355,33 +373,35 @@ class CmdContext_RtrBGP_AF_NbtAct(CmdContext_RtrBGP_AF):
 
     def parse(self, cfg, nbr):
         deepsetdefault(
-            cfg, "neighbor", neighbor_canonicalize(nbr))["activate"] = None
+            cfg, "neighbor", neighbor_canonicalize(nbr))["activate"] = True
 
 
 class CmdContext_RtrBGP_AF_NbrAddPath(CmdContext_RtrBGP_AF):
+    # 'receive' must come after 'send'; 'disable' exclusive
     match = (r"neighbor (?P<nbr>\S+) additional-paths"
              r"(( (?P<snd>send))?( (?P<rcv>receive))?|( (?P<dis>disable)))")
 
     def parse(self, cfg, nbr, snd, rcv, dis):
         # additional paths is a set of all matching types (or 'disable')
-        s = deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr))[
+        deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr))[
             "additional-paths"] = { a for a in (snd, rcv, dis) if a }
 
 
 class CmdContext_RtrBGP_AF_NbrAdvAddPath(CmdContext_RtrBGP_AF):
     match = (r"neighbor (?P<nbr>\S+) advertise additional-paths"
-             r"( (?P<all>all))?"
-             r"( (?P<best>best( (?P<best_n>\d+))))?"
-             r"( (?P<grp_best>group-best))?")
+             r"(?=.*\s(?P<all>all))?"
+             r"(?=.*\s(best( (?P<best_n>\d+))))?"
+             r"(?=.*\s(?P<grp_best>group-best))?"
+             r".+")
 
-    def parse(self, cfg, nbr, all, best, best_n, grp_best):
+    def parse(self, cfg, nbr, all, best_n, grp_best):
         a = deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr),
                            "advertise-additional-paths")
 
         if all:
             a["all"] = True
-        if best:
-            a["best"] = best_n
+        if best_n:
+            a["best"] = int(best_n)
         if grp_best:
             a["group-best"] = True
 
@@ -475,8 +495,8 @@ class CmdContext_RtrBGP_AF_NbrSoftRecfg(CmdContext_RtrBGP_AF):
                 "soft-reconfiguration"] = "inbound"
 
 
-class CmdContext_RtrBGP_AF_Redist(CmdContext_RtrBGP_AF):
-    match = (r"redistribute (?P<proto>static|connected|ospf \d+|ospfv3 \d+)"
+class CmdContext_RtrBGP_AF_Redist_Simple(CmdContext_RtrBGP_AF):
+    match = (r"redistribute (?P<proto>static|connected)"
              r"( route-map (?P<rtmap>\S+))?( metric (?P<met>\d+))?")
 
     def parse(self, cfg, proto, rtmap, met):
@@ -487,8 +507,86 @@ class CmdContext_RtrBGP_AF_Redist(CmdContext_RtrBGP_AF):
             r["metric"] = int(met)
 
 
+class CmdContext_RtrBGP_AF_Redist_ID(CmdContext_RtrBGP_AF):
+    match = (r"redistribute (?P<proto>ospf|ospfv3) (?P<proc>\d+)"
+             r"( route-map (?P<rtmap>\S+))?( metric (?P<met>\d+))?")
 
-# ROUTER OSPF ...
+    def parse(self, cfg, proto, proc, rtmap, met):
+        r = deepsetdefault(cfg, "redistribute", proto, int(proc))
+        if rtmap:
+            r["route-map"] = rtmap
+        if met:
+            r["metric"] = int(met)
+
+
+
+# router ospf ...
+
+
+
+# this function is used for both ospf and ospfv3 so is shared
+
+def _ospf_passive_interface(cfg, int_name, passive):
+    """This function maintains a "passive-interface" dictionary under
+    the supplied configuration (cfg) for OSPF or an OSPFv3 address
+    family.
+
+    The named interface (int_name) is set to either passive or active.
+
+    If the interface name is the literal "default" then the default
+    interface status is changed.
+    """
+
+    # create the "passive-interface" dictionary, if it doesn't exist
+    p = cfg.setdefault("passive-interface", {})
+
+
+    if int_name == "default":
+        # we're changing the default
+
+        if passive:
+            # we're making passive interfaces the default so we don't
+            # need to keep a list of passive interfaces, if there is one
+            p["default"] = True
+            if "interface" in p:
+                p.pop("interface")
+        else:
+            # the opposite of the above
+            if "default" in p:
+                p.pop("default")
+            if "no-interface" in p:
+                p.pop("no-interface")
+
+    else:
+        # we're changing an individual interface
+
+        # the name of the set of interfaces we're adjusting depends on
+        # whether this interface is being configured as passive or
+        # active
+        int_set_key = "interface" if passive else "no-interface"
+
+        ic = interface_canonicalize(int_name)
+        if passive != p.get("default", False):
+            # we're setting an interface state to the opposite of the
+            # default - add it to the exception list
+            deepsetdefault(p, int_set_key, last=set()).add(ic)
+
+        else:
+            # we're setting an interface to the same state as the
+            # default - remove it from the exception set, if it exists
+            if int_set_key in p:
+                p[int_set_key].discard(ic)
+
+                # if the set is now empty, we remove it completely
+                if not p.get(int_set_key):
+                    p.pop(int_set_key)
+
+
+    # if the passive interface configuration is now empty (which means
+    # we have active interfaces by default with no exceptions), remove
+    # the whole passive interface configuration dictionary
+    if not p:
+        cfg.pop("passive-interface")
 
 
 
@@ -504,7 +602,7 @@ class CmdContext_RtrOSPF(IndentedContextualCommand):
     context = "router-ospf"
 
 
-class Cmd_RtrOSPF_Id(CmdContext_RtrOSPF):
+class Cmd_RtrOSPF_ID(CmdContext_RtrOSPF):
     match = r"router-id (?P<id_>[.0-9]+)"
 
     def parse(self, cfg, id_):
@@ -517,27 +615,22 @@ class Cmd_RtrOSPF_AreaNSSA(CmdContext_RtrOSPF):
              r"(?P<no_summ> no-summary)?")
 
     def parse(self, cfg, area, no_redist, no_summ):
-        a = set()
-        if no_redist: a.add("no-redistribution")
-        if no_summ: a.add("no-summary")
-        deepsetdefault(cfg, "area", area)["nssa"] = a
+        n = deepsetdefault(cfg, "area", area, "nssa", last=set())
+        if no_redist:
+            n.add("no-redistribution")
+        if no_summ:
+            n.add("no-summary")
 
 
 class Cmd_RtrOSPF_PasvInt(CmdContext_RtrOSPF):
     match = r"(?P<no>no )?passive-interface (?P<int_name>\S+)"
 
     def parse(self, cfg, no, int_name):
-        p = deepsetdefault(cfg, "passive-interface")
-        if int_name == "default":
-            if not no:
-                p["default"] = True
-        else:
-            deepsetdefault(
-                p, "interface")[interface_canonicalize(int_name)] = not no
+        _ospf_passive_interface(cfg, int_name, not no)
 
 
 
-# ROUTER OSPFV3 ...
+# router ospfv3 ...
 
 
 
@@ -566,10 +659,11 @@ class Cmd_RtrOSPFv3_AreaNSSA(CmdContext_RtrOSPFv3):
              r"(?P<no_summ> no-summary)?")
 
     def parse(self, cfg, area, no_redist, no_summ):
-        a = set()
-        if no_redist: a.add("no-redistribution")
-        if no_summ: a.add("no-summary")
-        deepsetdefault(cfg, "area", area)["nssa"] = a
+        n = deepsetdefault(cfg, "area", area, "nssa", last=set())
+        if no_redist:
+            n.add("no-redistribution")
+        if no_summ:
+            n.add("no-summary")
 
 
 class Cmd_RtrOSPFv3_AF(CmdContext_RtrOSPFv3):
@@ -589,13 +683,7 @@ class Cmd_RtrOSPFv3_AF_PasvInt(CmdContext_RtrOSPFv3_AF):
     match = r"(?P<no>no )?passive-interface (?P<int_name>\S+)"
 
     def parse(self, cfg, no, int_name):
-        p = deepsetdefault(cfg, "passive-interface")
-        if int_name == "default":
-            if not no:
-                p["default"] = True
-        else:
-            deepsetdefault(
-                p, "interface")[interface_canonicalize(int_name)] = not no
+        _ospf_passive_interface(cfg, int_name, not no)
 
 
 class Cmd_RtrOSPFv3_PasvInt(CmdContext_RtrOSPFv3):
@@ -608,10 +696,5 @@ class Cmd_RtrOSPFv3_PasvInt(CmdContext_RtrOSPFv3):
         # currently defined address families, so if an address family
         # is added later, this will not propagate down
         for af in cfg.get("address-family", []):
-            p = deepsetdefault(cfg, "address-family", af, "passive-interface")
-            if int_name == "default":
-                if not no:
-                    p["default"] = True
-            else:
-                deepsetdefault(
-                    p, "interface")[interface_canonicalize(int_name)] = not no
+            _ospf_passive_interface(
+                cfg["address-family"][af], int_name, not no)
