@@ -260,7 +260,7 @@ class Cvt_Int_IPIGMPVer(DiffConvert_Int):
         return self.enter(int_name) + [" no ip igmp version"]
 
     def update(self, old, upd, new, int_name):
-        return self.enter(int_name) + [" ip igmp version " + new]
+        return self.enter(int_name) + [" ip igmp version %d" % new]
 
 
 class Cvt_Int_IPMcastBdry(DiffConvert_Int):
@@ -357,10 +357,10 @@ class Cvt_Int_IPPIMMode(DiffConvert_Int):
     cmd = "ip-pim", "mode"
 
     def remove(self, old, int_name):
-        return self.enter(int_name) + [" no ip pim " + old]
+        return self.enter(int_name) + [" no ip pim %s-mode" % old]
 
     def update(self, old, upd, new, int_name):
-        return self.enter(int_name) + [" ip pim " + new]
+        return self.enter(int_name) + [" ip pim %s-mode" % new]
 
 
 class Cvt_Int_IPPIMBSRBdr(DiffConvert_Int):
