@@ -10,6 +10,8 @@
 
 import netaddr
 
+from deepops import deepget
+
 from ...diff import DiffConvert
 
 
@@ -18,7 +20,7 @@ from ...diff import DiffConvert
 
 
 
-# IP[V6] ROUTE ...
+# ip[v6] route ...
 
 
 
@@ -33,8 +35,8 @@ class Cvt_IPRoute(DiffConvert):
                 + " " + str(n.network) + " " + str(n.netmask)
                 + ((" " + r["interface"]) if "interface" in r else "")
                 + ((" " + r["router"]) if "router" in r else "")
-                + ((" " + str(r["metric"])) if "metric" in r else "")
-                + ((" tag " + str(r["tag"])) if "tag" in r else ""))
+                + ((" %d" % r["metric"]) if "metric" in r else "")
+                + ((" tag %d" % r["tag"]) if "tag" in r else ""))
 
     def remove(self, old, vrf, net, id):
         return "no " + self._cmd(vrf, net, old)
@@ -63,7 +65,7 @@ class Cvt_IPv6Route(DiffConvert):
 
 
 
-# ROUTE-MAP ...
+# route-map ...
 
 
 
@@ -102,28 +104,52 @@ class DiffConvert_RtMap_Entry(DiffConvert_RtMap):
         return ["route-map %s %s %d" % (rtmap, rtmap_dict["action"], seq)]
 
 
-class Cvt_RtMap_MatchCmty(DiffConvert_RtMap_Entry):
+class Cvt_RtMap_MatchCmty_DelExact(DiffConvert_RtMap_Entry):
     cmd = tuple()
-    ext = "match", "community", "communities"
+    ext = "match", "community", "exact-match"
+    block = "rtmap-del"
+    trigger_blocks = { "rtmap-add-cmty" }
 
-class Cvt_RtMap_MatchCmty_Del(Cvt_RtMap_MatchCmty):
+    # if removing the exact-match option, we need to clear the list and
+    # recreate it without it
+    def remove(self, old, rtmap, seq):
+        if self.get_ext(old):
+            l = self.enter(rtmap, old, seq)
+            l.append(" no match community")
+            return l
+
+
+class DiffConvert_RtMap_MatchCmty(DiffConvert_RtMap_Entry):
+    cmd = tuple()
+    ext = "match", "community", "communities", None
+
+
+class Cvt_RtMap_MatchCmty_Del(DiffConvert_RtMap_MatchCmty):
     block = "rtmap-del"
 
-    def truncate(self, old, rem, new, rtmap, seq):
+    def truncate(self, old, rem, new, rtmap, seq, cmty):
         l = self.enter(rtmap, old, seq)
-        for cmty in self.get_ext(rem):
-            l.append(" no match community " + cmty)
+        l.append(" no match community " + cmty)
         return l
 
-class Cvt_RtMap_MatchCmty_add(Cvt_RtMap_MatchCmty):
-    block = "rtmap-add"
 
-    def update(self, old, upd, new, rtmap, seq):
+class Cvt_RtMap_MatchCmty_add(DiffConvert_RtMap_MatchCmty):
+    block = "rtmap-add-cmty"
+
+    def update(self, old, upd, new, rtmap, seq, cmty):
+        # TODO: need to handle applying 'exact-match' when list is the
+        # same; will rework these to use _context_args_offset to see if
+        # that makes it easier (as we need the action from a higher
+        # context but would like to set a lower context)
         l = self.enter(rtmap, new, seq)
-        for cmty in self.get_ext(upd):
-            l.append(" match community " + cmty)
+        exact_match = deepget(new, "match", "community", "exact-match")
+        l.append(" match community %s%s"
+                     % (cmty, " exact-match" if exact_match else ""))
         return l
 
+    def trigger(self, new, *args):
+        return self.update(new, new, new, *args)
+
 
 class Cvt_RtMap_MatchIPAddr(DiffConvert_RtMap_Entry):
     cmd = tuple()
@@ -302,6 +328,20 @@ class Cvt_RtMap_SetIPNxtHop_Add(Cvt_RtMap_SetIPNxtHop):
 
 
 class Cvt_RtMap_SetIPNxtHopVrfy(DiffConvert_RtMap_Entry):
+    cmd = tuple()
+    ext = "set", "ip-next-hop-verify-availability"
+
+    def remove(self, old, rtmap, seq):
+        l = self.enter(rtmap, new, seq)
+        l.append(" no set ip next-hop verify-availability")
+        return l
+
+    def update(self, old, upd, new, rtmap, seq):
+        l = self.enter(rtmap, new, seq)
+        l.append(" set ip next-hop verify-availability")
+        return l
+
+class Cvt_RtMap_SetIPNxtHopVrfyTrk(DiffConvert_RtMap_Entry):
     cmd = tuple()
     ext = "set", "ip-next-hop-verify-availability", None
 
@@ -309,7 +349,7 @@ class Cvt_RtMap_SetIPNxtHopVrfy(DiffConvert_RtMap_Entry):
         return ("set ip next-hop verify-availability %s %s track %d"
                      % (nexthop["addr"], seq, nexthop["track-obj"]))
 
-class Cvt_RtMap_SetIPNxtHopVrfy_Del(Cvt_RtMap_SetIPNxtHopVrfy):
+class Cvt_RtMap_SetIPNxtHopVrfy_Del(Cvt_RtMap_SetIPNxtHopVrfyTrk):
     block = "rtmap-del"
 
     def remove(self, old, rtmap, seq, nexthop_seq):
@@ -318,7 +358,7 @@ class Cvt_RtMap_SetIPNxtHopVrfy_Del(Cvt_RtMap_SetIPNxtHopVrfy):
                        + self._cmd(nexthop_seq,
                                    self.get_ext(old, nexthop_seq))]
 
-class Cvt_RtMap_SetIPNxtHopVrfy_Add(Cvt_RtMap_SetIPNxtHopVrfy):
+class Cvt_RtMap_SetIPNxtHopVrfy_Add(Cvt_RtMap_SetIPNxtHopVrfyTrk):
     block = "rtmap-add"
 
     def update(self, old, upd, new, rtmap, seq, nexthop_seq):
@@ -344,7 +384,7 @@ class Cvt_RtMap_SetIPv6NxtHop_Del(Cvt_RtMap_SetIPv6NxtHop):
     block = "rtmap-del"
 
     def remove(self, old, rtmap, seq):
-        # we must remove all the 'set ip next-hop' commands individually
+        # we must remove all the 'set ipv6 next-hop' commands individually
         l = self.enter(rtmap, old, seq)
         for nexthop in self.get_ext(old):
             l.append(" no " + self._cmd(nexthop))
@@ -366,6 +406,52 @@ class Cvt_RtMap_SetIPv6NxtHop_Add(Cvt_RtMap_SetIPv6NxtHop):
         return l
 
 
+class Cvt_RtMap_SetIPv6NxtHopVrfy(DiffConvert_RtMap_Entry):
+    cmd = tuple()
+    ext = "set", "ipv6-next-hop-verify-availability"
+
+    def remove(self, old, rtmap, seq):
+        l = self.enter(rtmap, new, seq)
+        l.append(" no set ipv6 next-hop verify-availability")
+        return l
+
+    def update(self, old, upd, new, rtmap, seq):
+        l = self.enter(rtmap, new, seq)
+        l.append(" set ipv6 next-hop verify-availability")
+        return l
+
+
+class Cvt_RtMap_SetIPNxtHopVrfyTrk(DiffConvert_RtMap_Entry):
+    cmd = tuple()
+    ext = "set", "ipv6-next-hop-verify-availability", None
+
+    def _cmd(self, seq, nexthop):
+        return ("set ipv6 next-hop verify-availability %s %s track %d"
+                     % (nexthop["addr"], seq, nexthop["track-obj"]))
+
+class Cvt_RtMap_SetIPv6NxtHopVrfyTrk_Del(Cvt_RtMap_SetIPNxtHopVrfyTrk):
+    block = "rtmap-del"
+
+    def remove(self, old, rtmap, seq, nexthop_seq):
+        return self.enter(rtmap, old, seq) + [
+                   " no "
+                       + self._cmd(nexthop_seq,
+                                   self.get_ext(old, nexthop_seq))]
+
+class Cvt_RtMap_SetIPv6NxtHopVrfyTrk_Add(Cvt_RtMap_SetIPNxtHopVrfyTrk):
+    block = "rtmap-add"
+
+    def update(self, old, upd, new, rtmap, seq, nexthop_seq):
+        # individual entries (ordered by sequence number) can be replaced but
+        # the old entry must be removed first, before the new one added
+        l = self.enter(rtmap, new, seq)
+        if old:
+            l.append(" no "
+                     + self._cmd(nexthop_seq, self.get_ext(old, nexthop_seq)))
+        l.append(" " + self._cmd(nexthop_seq, self.get_ext(new, nexthop_seq)))
+        return l
+
+
 class Cvt_RtMap_SetLocalPref(DiffConvert_RtMap_Entry):
     cmd = tuple()
     ext = "set", "local-preference"
@@ -411,324 +497,7 @@ class Cvt_RtMap_SetVRF_Add(Cvt_RtMap_SetVRF):
 
 
 
-# ROUTER BGP ...
-
-
-
-class Cvt_RtrBGP(DiffConvert):
-    cmd = "router", "bgp", None
-
-    def remove(self, old, asn):
-        return "no router bgp " + str(asn)
-
-    def add(self, new, asn):
-        return "router bgp " + str(asn)
-
-
-class DiffConvert_RtrBGP(DiffConvert):
-    context = "router", "bgp", None
-
-    def enter(self, asn):
-        return ["router bgp " + str(asn)]
-
-
-class Cvt_RtrBGP_BGPRtrID(DiffConvert_RtrBGP):
-    cmd = "router-id",
-
-    def remove(self, old, asn):
-        return self.enter(asn) + [" no bgp router-id"]
-
-    def update(self, old, upd, new, asn):
-        return self.enter(asn) + [" bgp router-id " + new]
-
-
-class Cvt_RtrBGP_Nbr(DiffConvert_RtrBGP):
-    cmd = "neighbor", None
-
-    def remove(self, old, *args):
-        c, (nbr, ) = self._context_args(args)
-        # when removing a neighbor that is a peer-group, we need to
-        # state that
-        return self.enter(*c) + [
-                   "  no neighbor %s%s"
-                       % (nbr,
-                          " peer-group" if old.get("type") == "peer-group"
-                              else "")]
-
-    def add(self, new, *args):
-        c, (nbr, ) = self._context_args(args)
-        # we only explicitly need to add a neighbor if it's a peer-group
-        # (normally, a neighbor is created implicitly by configuring
-        # settings for it)
-        if new.get("type") == "peer-group":
-            return self.enter(*c) + ["  neighbor %s peer-group" % nbr]
-
-
-class DiffConvert_RtrBGP_Nbr(DiffConvert_RtrBGP):
-    context = DiffConvert_RtrBGP.context + Cvt_RtrBGP_Nbr.cmd
-
-    # we don't want the neighbor as part of the context
-    _context_args_offset = -1
-
-
-class Cvt_RtrBGP_Nbr_FallOver(DiffConvert_RtrBGP_Nbr):
-    cmd = "fall-over",
-
-    def remove(self, old, asn, nbr):
-        return self.enter(asn) + [" no neighbor %s fall-over"]
-
-    def update(self, old, upd, new, asn, nbr):
-        return self.enter(asn) + [
-                   " neighbor %s fall-over %s"
-                       % (nbr,
-                          ("bfd " + new["bfd"]) if "bfd" in new
-                               else "route-map " + new["route-map"])]
-
-
-class Cvt_RtrBGP_Nbr_Pwd(DiffConvert_RtrBGP_Nbr):
-    cmd = "password",
-
-    def remove(self, old, asn, nbr):
-        return self.enter(asn) + [" no neighbor %s password"]
-
-    def update(self, old, upd, new, asn, nbr):
-        return self.enter(asn) + [
-                   " neighbor %s password encryption %d %s"
-                       % (nbr, new["encyrption"], new["password"])]
-
-
-class Cvt_RtrBGP_Nbr_PrGrpMbr(DiffConvert_RtrBGP_Nbr):
-    # this converter is used to add or remove a neighbor to/from a
-    # peer-group
-    cmd = "peer-group",
-
-    def remove(self, old, *args):
-        c, (nbr, ) = self._context_args(args)
-        return self.enter(*c) + [" no neighbor %s peer-group" % nbr]
-
-    def update(self, old, upd, new, *args):
-        c, (nbr, ) = self._context_args(args)
-        return self.enter(*c) + [" neighbor %s peer-group %s" % (nbr, new)]
-
-
-class Cvt_RtrBGP_Nbr_RemAS(DiffConvert_RtrBGP_Nbr):
-    cmd = "remote-as",
-
-    def remove(self, old, asn, nbr):
-        return self.enter(asn) + [" no neighbor %s remote-as"]
-
-    def update(self, old, upd, new, asn, nbr):
-        return self.enter(asn) + [" neighbor %s remote-as %s" % (nbr, new)]
-
-
-class Cvt_RtrBGP_Nbr_UpdSrc(DiffConvert_RtrBGP_Nbr):
-    cmd = "update-source",
-
-    def remove(self, old, asn, nbr):
-        return self.enter(asn) + [" no neighbor %s update-source" % nbr]
-
-    def update(self, old, upd,new, asn, nbr):
-        return self.enter(asn) + [" neighbor %s update-source %s" % (nbr, new)]
-
-
-# router bgp ... address-family ... [vrf ...]
-
-
-# working out the address-family line is complicated and we do it in
-# several places, so separate it into a function
-
-def _RtrBGP_AF(vrf, af):
-    # address families in the global routing table as in a VRF called
-    # '_default' as a special case
-    return (" address-family %s%s"
-                % (af, (" vrf " + vrf) if vrf != "_default" else ""))
-
-
-class Cvt_RtrBGP_AF(DiffConvert_RtrBGP):
-    cmd = "vrf", None, "address-family", None
-
-    def remove(self, old, asn, vrf, af):
-        return self.enter(asn) + [" no" + _RtrBGP_AF(vrf, af)]
-
-    def add(self, new, asn, vrf, af):
-        return self.enter(asn) + [_RtrBGP_AF(vrf, af)]
-
-
-
-class DiffConvert_RtrBGP_AF(DiffConvert_RtrBGP):
-    context = DiffConvert_RtrBGP.context + Cvt_RtrBGP_AF.cmd
-
-    def enter(self, asn, vrf, af):
-        return super().enter(asn) + [_RtrBGP_AF(vrf, af)]
-
-
-class Cvt_RtrBGP_AF_Nbr(DiffConvert_RtrBGP_AF, Cvt_RtrBGP_Nbr):
-    pass
-
-
-class DiffConvert_RtrBGP_AF_Nbr(DiffConvert_RtrBGP_AF):
-    context = DiffConvert_RtrBGP_AF.context + Cvt_RtrBGP_AF_Nbr.cmd
-
-
-class Cvt_RtrBGP_AF_Nbr_Act(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "activate",
-
-    def remove(self, old, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + ["  no neighbor %s activate" % nbr]
-
-    def add(self, new, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + ["  neighbor %s activate" % nbr]
-
-
-class Cvt_RtrBGP_AF_Nbr_AddPath(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "additional-paths",
-
-    def _add_paths(self, p):
-        return (
-            " ".join([a for a in [ "send", "receive", "disable" ] if a in p]))
-
-    def remove(self, old, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s additional-paths" % nbr]
-
-    def truncate(self, old, rem, new, asn, vrf, af, nbr):
-        # we can't remove types of additional-path, only provide a
-        # complete new list
-        return self.update(old, None, new, asn, vrf, af, nbr)
-
-    def update(self, old, upd, new, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s additional-paths %s"
-                       % (nbr, self._add_paths(new))]
-
-
-class Cvt_RtrBGP_AF_Nbr_AdvAddPath(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "advertise-additional-paths", None
-
-    def remove(self, old, asn, vrf, af, nbr, adv):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s advertise additional-paths %s"
-                       % (nbr, adv)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr, adv):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s advertise additional-paths %s"
-                       % (nbr, ("best %d" % new) if adv == "best" else adv)]
-
-
-class Cvt_RtrBGP_AF_Nbr_FltLst(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "filter-list", None
-
-    def remove(self, old, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s filter-list %d %s" % (nbr, old, dir_)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s filter-list %d %s" % (nbr, new, dir_)]
-
-
-class Cvt_RtrBGP_AF_Nbr_PrGrpMbr(
-          DiffConvert_RtrBGP_AF_Nbr, Cvt_RtrBGP_Nbr_PrGrpMbr):
-
-    pass
-
-
-class Cvt_RtrBGP_AF_Nbr_Pfx(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "prefix-list", None
-
-    def remove(self, old, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s prefix-list %s %s" % (nbr, old, dir_)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s prefix-list %s %s" % (nbr, new, dir_)]
-
-
-class Cvr_RtrBGP_AF_Nbr_RtMap(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "route-map", None
-
-    def remove(self, old, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s route-map %s %s" % (nbr, old, dir_)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr, dir_):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s route-map %s %s" % (nbr, new, dir_)]
-
-
-class Cvt_RtrBGP_AF_Nbr_SndCmty(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "send-community", None
-
-    # the 'neighbor ... send-community' command is odd in that the
-    # 'standard', 'extended' and 'both' options don't replace the
-    # current setting but add or remove those communities to it
-    #
-    # the configuration is expressed as a set containing none, one or
-    # both of 'standard' and 'extended'
-
-    def remove(self, old, asn, vrf, af, nbr, cmty):
-       return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s send-community %s" % (nbr, cmty)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr, cmty):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s send-community %s" % (nbr, cmty)]
-
-
-class Cvt_RtrBGP_AF_Nbr_SoftRecfg(DiffConvert_RtrBGP_AF_Nbr):
-    cmd = "soft-reconfiguration",
-
-    def remove(self, old, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + [
-                   "  no neighbor %s soft-reconfiguration %s" % (nbr, old)]
-
-    def update(self, old, upd, new, asn, vrf, af, nbr):
-        return self.enter(asn, vrf, af) + [
-                   "  neighbor %s soft-reconfiguration %s" % (nbr, new)]
-
-
-class Cvt_RtrBGP_AF_Redist(DiffConvert_RtrBGP_AF):
-    cmd = "redistribute", None
-
-    def remove(self, old, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + ["  no redistribute " + proto]
-
-    def add(self, new, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + ["  redistribute " + proto]
-
-
-class DiffConvert_RtrBGP_AF_Redist(DiffConvert_RtrBGP_AF):
-    context = DiffConvert_RtrBGP_AF.context + Cvt_RtrBGP_AF_Redist.cmd
-
-
-class Cvt_RtrBGP_AF_Redist_RtMap(DiffConvert_RtrBGP_AF_Redist):
-    cmd = "route-map",
-
-    def remove(self, old, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + [
-                   "  no redistribute %s route-map %s" % (proto, old)]
-
-    def update(self, old, upd, new, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + [
-                   "  redistribute %s route-map %s" % (proto, new)]
-
-
-class Cvt_RtrBGP_AF_Redist_Metric(DiffConvert_RtrBGP_AF_Redist):
-    cmd = "metric",
-
-    def remove(self, old, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + [
-                   "  no redistribute %s metric" % proto]
-
-    def update(self, old, upd, new, asn, vrf, af, proto):
-        return self.enter(asn, vrf, af) + [
-                   "  redistribute %s metric %d" % (proto, new)]
-
-
-
-# ROUTER OSPF ...
+# router ospf ...
 
 
 
@@ -772,34 +541,55 @@ class Cvt_RtrOSPF_AreaNSSA(DiffConvert_RtrOSPF):
         return self.enter(proc) + [" area %s nssa%s" % (area, s)]
 
 
+# passive-interface configuration is slightly odd as the default mode is
+# stored (assuming it's not the default) and then a list of exceptions
+# is maintained and it can go either way
+
+
 class Cvt_RtrOSPF_PasvInt_Dflt(DiffConvert_RtrOSPF):
     cmd = "passive-interface", "default"
 
     def remove(self, old, proc):
-        return self.enter(proc) + [
-                   " %spassive-interface default" % ("no " if old else "")]
+        return self.enter(proc) + [" no passive-interface default"]
 
     def update(self, old, upd, new, proc):
-        return self.enter(proc) + [
-                   " %spassive-interface default" % ("" if new else "no ")]
+        return self.enter(proc) + [" passive-interface default"]
 
 
+# ... the exception interface lists must execute after changing the
+# default mode, which they will do as 'default' comes before 'interface'
+# and 'no-interface'
+
 class Cvt_RtrOSPF_PasvInt_Int(DiffConvert_RtrOSPF):
-    cmd = "passive-interface", "interface", None
+    cmd = "passive-interface",
+    ext = "interface", None
 
-    def remove(self, old, proc, int_name):
-        return self.enter(proc) + [
-                   " %spassive-interface %s"
-                       % ("no " if old else "", int_name)]
+    def delete(self, old, rem, new, proc, int_name):
+        # if we're changing the default mode, the old list of exceptions
+        # will be removed by that, so we don't need to do it
+        if (old or {}).get("default") == (new or {}).get("default"):
+            return self.enter(proc) + [" no passive-interface " + int_name]
 
     def update(self, old, upd, new, proc, int_name):
-        return self.enter(proc) + [
-                   " %spassive-interface %s"
-                       % ("" if new else "no ", int_name)]
+        return self.enter(proc) + [" passive-interface " + int_name]
+
+
+class Cvt_RtrOSPF_PasvInt_NoInt(DiffConvert_RtrOSPF):
+    cmd = "passive-interface",
+    ext = "no-interface", None
+
+    def delete(self, old, rem, new, proc, int_name):
+        # if we're changing the default mode, the old list of exceptions
+        # will be removed by that, so we don't need to do it
+        if (old or {}).get("default") == (new or {}).get("default"):
+            return self.enter(proc) + [" passive-interface " + int_name]
+
+    def update(self, old, upd, new, proc, int_name):
+        return self.enter(proc) + [" no passive-interface " + int_name]
 
 
 
-# ROUTER OSPFV3 ...
+# router ospfv3 ...
 
 
 
@@ -860,27 +650,39 @@ class DiffConvert_RtrOSPFv3_AF(DiffConvert_RtrOSPFv3):
         return super().enter(vrf) + [" address-family " + af]
 
 
+# see the Cvt_RtrOSPF_... versions above for the explanation of how
+# these converters work
+
+
 class Cvt_RtrOSPFv3_AF_PasvInt_Dflt(DiffConvert_RtrOSPFv3_AF):
     cmd = "passive-interface", "default"
 
     def remove(self, old, proc, af):
-        return self.enter(proc, af) + [
-                   " %spassive-interface default" % ("no " if old else "")]
+        return self.enter(proc, af) + [" no passive-interface default"]
 
     def update(self, old, upd, new, proc, af):
-        return self.enter(proc, af) + [
-                   "  %spassive-interface default" % ("" if new else "no ")]
+        return self.enter(proc, af) + [" passive-interface default"]
 
 
 class Cvt_RtrOSPFv3_AF_PasvInt_Int(DiffConvert_RtrOSPFv3_AF):
-    cmd = "passive-interface", "interface", None
+    cmd = "passive-interface",
+    ext = "interface", None
 
-    def remove(self, old, proc, af, int_name):
-        return self.enter(proc, af) + [
-                   "  %spassive-interface %s"
-                       % ("no " if old else "", int_name)]
+    def delete(self, old, rem, new, proc, af, int_name):
+        if (old or {}).get("default") == (new or {}).get("default"):
+            return self.enter(proc, af) + [" no passive-interface " + int_name]
 
     def update(self, old, upd, new, proc, af, int_name):
-        return self.enter(proc, af) + [
-                   "  %spassive-interface %s"
-                       % ("" if new else "no ", int_name)]
+        return self.enter(proc, af) + [" passive-interface " + int_name]
+
+
+class Cvt_RtrOSPFv3_AF_PasvInt_NoInt(DiffConvert_RtrOSPFv3_AF):
+    cmd = "passive-interface",
+    ext = "no-interface", None
+
+    def delete(self, old, rem, new, proc, af, int_name):
+        if (old or {}).get("default") == (new or {}).get("default"):
+            return self.enter(proc, af) + [" passive-interface " + int_name]
+
+    def update(self, old, upd, new, proc, af, int_name):
+        return self.enter(proc, af) + [" no passive-interface " + int_name]
