@@ -36,13 +36,14 @@ from ..config import IndentedContextualCommand, IndentedContextualConfig
 _INTERFACE_CANONICALS = {
     "ethernet": "Eth",
     "fastethernet": "Fa",
+    "fortygigabitethernet": "Fo",
     "gigabitethernet": "Gi",
+    "hundredgigethernet": "Hu",
+    "loopback": "Lo",
+    "port-channel": "Po",
     "tengigabitethernet": "Te",
     "twentyfivegigethernet": "Twe",
-    "fortygigabitethernet": "Fo",
-    "hundredgigethernet": "Hu",
-    "port-channel": "Po",
-    "vl": "Vlan",
+    "vlan": "Vl",
 }
 
 
@@ -148,6 +149,20 @@ def _ip_acl_std_rule_parse(rule):
     return match.group("action"), ipnet4
 
 
+def seq_to_list(s):
+    """Converts a dictionary keyed on sequence numbers (or anything
+    which can be sorted) into a list with the dictionary values in the
+    order of sequence nubmers.
+
+    This is used to turn a access lists with rules that have sequence
+    numbers into a plain list of rules, in the correct order but without
+    the numbers.  This is done after parsing access lists (and other
+    similar sequenced structures like prefix lists) into an ordered
+    list of rules.
+    """
+
+    return [s[e] for e in sorted(s)]
+
 
 def ip_acl_std_canonicalize(l):
     """This function sorts an IPv4 standard access-list into a canoncial
@@ -675,14 +690,15 @@ def explain_diffs(old, new, indent="", prefix="", to_str_func=None):
 
 
 
-# regular expression for matching physical interface names
+# regular expression for matching physical interface names; this assumes
+# the interface name as already been through interface_canonicalize()
 #
 # note that it will only match interfaces of the form
-# "<type><num>[/<num2>[/<num3>[/...]]]" and this excludes subinterfaces,
-# where the final number with have a dot in it
+# "<type><num>[/<num2>[/<num3>[/...]]]", deliberately excluding
+# subinterfaces, where the final number with have a dot before it
 
-_INTERFACE_PHYSICAL_RE = re.compile("^(Mgmt|Eth|Fa|Gi|Te|Twe|Fo|Hu)\d+(/\d+)*$",
-                                    re.IGNORECASE)
+_INTERFACE_PHYSICAL_RE = (
+    re.compile("^(Eth|Fa|Fo|Gi|Hu|Mgmt|Te|Twe)\d+(/\d+)*$"))
 
 
 def is_int_physical(i):
